#!/usr/bin/env bash
#
# Install this agent as a system service and set to start on boot.

HERE="$(readlink -f $(dirname $0))"
PREFIX="/opt/eval/manageddeviceagent"
JAR="ManagedDeviceAgent-2.1.0-all.jar"


# This installation requires root.
if [ "$(whoami)" != "root" ]; then
	echo "Please run this script as root!"
	exit 1
fi


# Parse stdin parameters
INTERACTIVE=1
SERVERURL="AUTO"

while [[ $# -gt 0 ]]; do
	case "$1" in
		--noninteractive)
			INTERACTIVE=0
			shift # single, no argument value
			;;
		--server)
			SERVERURL="$2"
			shift
			shift # Argument with value, skip past both
			;;
		*)
			# Unknown argument, skip
			shift
			;;
	esac
done


# Download a new version!
echo "Retrieving basescript bootstrap script..."
if [ ! -e "/opt/eval" ]; then
	mkdir -p "/opt/eval"
fi
wget -q https://mirror.eval.bz/eval.basescript/basescript.sh -O "/opt/eval/basescript.sh"
echo "Loading basescript..."
source "/opt/eval/basescript.sh"


if [ "$INTERACTIVE" == "1" ]; then
	printheader ""
	printline "By continuing with this application, you agree to allow your"
	printline "managed service provider to remotely execute commands"
	printline "on this local computer in order to provide support,"
	printline "maintenance, and manage security issues."
	printheader ""
	echo -n "Press [ENTER] to continue"
	read TRASH
fi



if [ -e "$PREFIX" ]; then
	printheader "Cleaning up previous install"

	systemctl stop manageddeviceagent

	if [ -e "/var/log/manageddeviceagent" ]; then
		rm -fr /var/log/manageddeviceagent
	fi

	if [ -e "/etc/logrotate.d/manageddeviceagent" ]; then
		rm -fr /etc/logrotate.d/manageddeviceagent
	fi

	if [ -e "/etc/init.d/manageddeviceagent" ]; then
		rm -fr /etc/init.d/manageddeviceagent
	fi

	if [ -e "/etc/rc.d/manageddeviceagent" ]; then
		rm -fr /etc/rc.d/manageddeviceagent
	fi
fi



printheader "Setting up directory structure"

# Setup the directory structure.
safemkdir "$PREFIX/bin"
safemkdir "$PREFIX/etc"
safemkdir "$PREFIX/lib"



printheader "Installing dependencies"

if [ "$OSFAMILY" == "debian" ]; then
	# Raspian "lite" doesn't ship with dirmngr, which is critical for using apt-key.
	if [ -z "$(which dirmngr 2>/dev/null)" ]; then
		install dirmngr
	fi
fi

if [ "$JAVAVERSIONMAJ" -lt 8 ]; then
	install jre-headless
fi

# Ensure a few pre-requirements are installed.
install python3-dateutil curl dmidecode smartmontools python3-six

# Copy the jar into the bin directory for this project.
cp "${HERE}/resources/${JAR}" "$PREFIX/bin/ManagedDeviceAgent.jar"


if [ ! -e "$PREFIX/etc/system.properties" ]; then
	if [ "$INTERACTIVE" == "1" ]; then
		printheader ""
		printline "A management server URL is required for this agent."
		printline "If you do not know this, please contact your MSP."
		printheader ""
		echo -n "Press enter the management URL: "
		read SERVERURL

		if [ "${SERVERURL/:\/\/}" == "${SERVERURL}" ]; then
			SERVERURL="https://${SERVERURL}"
		fi

		if [ "${SERVERURL/https:\/\/}" == "${SERVERURL}" ]; then
			SSL="0"
		else
			SSL="1"
		fi


		printheader ""
		printline "For privacy concerns, remote desktop access requires"
		printline "user approval prior to a technician connecting."
		printline ""
		printline "Unattended setups for servers can be enabled if necessary."
		printheader ""
		echo -n "Enable unattended access? (y/N): "
		read UNATTENDED

		case "$UNATTENDED" in
			"y" | "Y" | "yes" | "YES")
				UNATTENDED="1"
				;;
			*)
				UNATTENDED="0"
		esac

		SERVERURL="${SERVERURL/:/\\:}"

		echo "server=${SERVERURL}" >> "$PREFIX/etc/system.properties"
		echo "ssl=${SSL}" >> "$PREFIX/etc/system.properties"
		echo "unattended=${UNATTENDED}" >> "$PREFIX/etc/system.properties"
	else
		# Try to auto-create
		if [ "$SERVERURL" == "AUTO" ]; then
			printwarning "Installing in non-interactive mode, but no server set!"
			printline "You will not be able to start the agent until a server is defined"
			printline " in $PREFIX/etc/system.properties"
		else
			if [ "${SERVERURL/https:\/\/}" == "${SERVERURL}" ]; then
				SSL="0"
			else
				SSL="1"
			fi

			echo "server=${SERVERURL}" >> "$PREFIX/etc/system.properties"
			echo "ssl=${SSL}" >> "$PREFIX/etc/system.properties"
			echo "unattended=1" >> "$PREFIX/etc/system.properties"
		fi
	fi
fi



printheader "Installing and starting service"
cp "$HERE/resources/manageddeviceagent.service" "/etc/systemd/system/manageddeviceagent.service"
systemctl daemon-reload
systemctl enable manageddeviceagent
systemctl start manageddeviceagent



printsuccess "Management Agent Installed!"
